@@ -115,6 +115,12 @@ class Accessory:
         self.add_service(service)
         return service
 
+    def set_primary_service(self, primary_service):
+        """Set the primary service of the acc."""
+        for service in self.services:
+            service.is_primary_service = service.type_id == \
+                primary_service.type_id
+
     def config_changed(self):
         """Notify the accessory about configuration changes.
 
@@ -128,12 +134,11 @@ class Accessory:
            (i.e. an Accessory that is contained in a Bridge),
            you should call the `config_changed` method on the Bridge.
 
-        Deprecated. Use `driver.state_change()` instead.
+        Deprecated. Use `driver.config_changed()` instead.
         """
         logger.warning(
-            'This method is now deprecated. Use \' '
-            'driver.state_version\' instead.')
-        self.driver.state_changed()
+            'This method is now deprecated. Use \'driver.config_changed\' instead.')
+        self.driver.config_changed()
 
     def add_service(self, *servs):
         """Add the given services to this Accessory.
