@@ -23,6 +23,7 @@ AccessoryDriver.
 """
 import asyncio
 from concurrent.futures import ThreadPoolExecutor
+import functools
 import os
 import logging
 import socket
@@ -47,6 +48,7 @@ from pyhap.hsrp import Server as SrpServer
 from pyhap.loader import Loader
 from pyhap.params import get_srp_context
 from pyhap.state import State
+from pyhap import util
 
 logger = logging.getLogger(__name__)
 
@@ -65,21 +67,28 @@ def is_callback(func):
     return '_pyhap_callback' in getattr(func, '__dict__', {})
 
 
+def iscoro(func):
+    """Check if the function is a coroutine or if the function is a ``functools.patial``,
+    check the wrapped function for the same.
+    """
+    if isinstance(func, functools.partial):
+        func = func.func
+    return asyncio.iscoroutinefunction(func)
+
+
 class AccessoryMDNSServiceInfo(ServiceInfo):
     """A mDNS service info representation of an accessory."""
 
     def __init__(self, accessory, state):
         self.accessory = accessory
         self.state = state
-        hname = socket.gethostname()
-        pubname = hname + '.' if hname.endswith('.local') else hname + '.local.'
 
         adv_data = self._get_advert_data()
         super().__init__(
             '_hap._tcp.local.',
             self.accessory.display_name + '._hap._tcp.local.',
             socket.inet_aton(self.state.address), self.state.port,
-            0, 0, adv_data, pubname)
+            0, 0, adv_data)
 
     def _setup_hash(self):
         setup_hash_material = self.state.setup_id + self.state.mac
@@ -116,10 +125,13 @@ class AccessoryDriver:
     """
 
     NUM_EVENTS_BEFORE_STATS = 100
+    """Number of HAP send events to be processed before reporting statistics on
+    the event queue length."""
 
     def __init__(self, *, address=None, port=51234,
                  persist_file='accessory.state', pincode=None,
-                 encoder=None, loader=None, loop=None):
+                 encoder=None, loader=None, loop=None, mac=None,
+                 listen_address=None, advertised_address=None):
         """
         Initialize a new AccessoryDriver object.
 
@@ -145,18 +157,31 @@ class AccessoryDriver:
 
         :param encoder: The encoder to use when persisting/loading the Accessory state.
         :type encoder: AccessoryEncoder
+
+        :param mac: The MAC address which will be used to identify the accessory.
+            If not given, the driver will try to select a MAC address.
+        :type mac: str
+
+        :param listen_address: The local address on the HAPServer will listen.
+            If not given, the value of the address parameter will be used.
+        :type listen_address: str
+
+        :param advertised_address: The address of the HAPServer announced via mDNS.
+            This can be used to announce an external address from behind a NAT.
+            If not given, the value of the address parameter will be used.
+        :type advertised_address: str
         """
         if sys.platform == 'win32':
             self.loop = loop or asyncio.ProactorEventLoop()
         else:
             self.loop = loop or asyncio.new_event_loop()
 
-        executer_opts = {'max_workers': None}
+        executor_opts = {'max_workers': None}
         if sys.version_info >= (3, 6):
-            executer_opts['thread_name_prefix'] = 'SyncWorker'
+            executor_opts['thread_name_prefix'] = 'SyncWorker'
 
-        self.executer = ThreadPoolExecutor(**executer_opts)
-        self.loop.set_default_executor(self.executer)
+        self.executor = ThreadPoolExecutor(**executor_opts)
+        self.loop.set_default_executor(self.executor)
 
         self.accessory = None
         self.http_server_thread = None
@@ -173,12 +198,17 @@ class AccessoryDriver:
         self.sent_events = 0
         self.accumulated_qsize = 0
 
+        self.safe_mode = False
+
         self.mdns_service_info = None
         self.srp_verifier = None
-        self.accessory_thread = None
 
-        self.state = State(address=address, pincode=pincode, port=port)
-        network_tuple = (self.state.address, self.state.port)
+        address = address or util.get_local_address()
+        advertised_address = advertised_address or address
+        self.state = State(address=advertised_address, mac=mac, pincode=pincode, port=port)
+
+        listen_address = listen_address or address
+        network_tuple = (listen_address, self.state.port)
         self.http_server = HAPServer(network_tuple, self)
 
     def start(self):
@@ -188,9 +218,18 @@ class AccessoryDriver:
         """
         try:
             logger.info('Starting the event loop')
+            if threading.current_thread() is threading.main_thread():
+                logger.debug('Setting child watcher')
+                watcher = asyncio.SafeChildWatcher()
+                watcher.attach_loop(self.loop)
+                asyncio.set_child_watcher(watcher)
+            else:
+                logger.debug('Not setting a child watcher. Set one if '
+                             'subprocesses will be started outside the main thread.')
             self.add_job(self._do_start)
             self.loop.run_forever()
         except KeyboardInterrupt:
+            logger.debug('Got a KeyboardInterrupt, stopping driver')
             self.loop.call_soon_threadsafe(
                 self.loop.create_task, self.async_stop())
             self.loop.run_forever()
@@ -213,7 +252,7 @@ class AccessoryDriver:
         if self.accessory is None:
             raise ValueError("You must assign an accessory to the driver, "
                              "before you can start it.")
-        logger.info("Starting accessory %s on address %s, port %s.",
+        logger.info('Starting accessory %s on address %s, port %s.',
                     self.accessory.display_name, self.state.address,
                     self.state.port)
 
@@ -224,14 +263,17 @@ class AccessoryDriver:
         #   finished, it will check the run sentinel, see that it is set and break the
         #   loop. Alternatively, the server's server_close method will shutdown and close
         #   the socket, while sending is in progress, which will result abort the sending.
+        logger.debug('Starting event thread.')
         self.send_event_thread = threading.Thread(daemon=True, target=self.send_events)
         self.send_event_thread.start()
 
         # Start listening for requests
+        logger.debug('Starting server.')
         self.http_server_thread = threading.Thread(target=self.http_server.serve_forever)
         self.http_server_thread.start()
 
         # Advertise the accessory as a mDNS service.
+        logger.debug('Starting mDNS.')
         self.mdns_service_info = AccessoryMDNSServiceInfo(
             self.accessory, self.state)
         self.advertiser.register_service(self.mdns_service_info)
@@ -241,6 +283,7 @@ class AccessoryDriver:
             self.accessory.setup_message()
 
         # Start the accessory so it can do stuff.
+        logger.debug('Starting accessory.')
         self.add_job(self.accessory.run)
         logger.debug('AccessoryDriver started successfully')
 
@@ -252,9 +295,10 @@ class AccessoryDriver:
     async def async_stop(self):
         """Stops the AccessoryDriver and shutdown all remaining tasks."""
         await self.async_add_job(self._do_stop)
-        logger.debug('Shutdown executers')
-        self.executer.shutdown()
+        logger.debug('Shutdown executors')
+        self.executor.shutdown()
         self.loop.stop()
+        logger.debug('Stop completed')
 
     def _do_stop(self):
         """Stop the accessory.
@@ -286,7 +330,7 @@ class AccessoryDriver:
         logger.debug("AccessoryDriver stopped successfully")
 
     def add_job(self, target, *args):
-        """Add job to executer pool."""
+        """Add job to executor pool."""
         if target is None:
             raise ValueError("Don't call add_job with None.")
         self.loop.call_soon_threadsafe(self.async_add_job, target, *args)
@@ -300,7 +344,7 @@ class AccessoryDriver:
             task = self.loop.create_task(target)
         elif is_callback(target):
             self.loop.call_soon(target, *args)
-        elif asyncio.iscoroutinefunction(target):
+        elif iscoro(target):
             task = self.loop.create_task(target(*args))
         else:
             task = self.loop.run_in_executor(None, target, *args)
@@ -398,10 +442,12 @@ class AccessoryDriver:
             # order to increase throughput.
             topic, bytedata = self.event_queue.get()
             subscribed_clients = self.topics.get(topic, [])
+            logger.debug('Send event: topic(%s), data(%s)', topic, bytedata)
             for client_addr in subscribed_clients.copy():
+                logger.debug('Sending event to client: %s', client_addr)
                 pushed = self.http_server.push_event(bytedata, client_addr)
                 if not pushed:
-                    logger.debug("Could not send event to %s, probably stale socket.",
+                    logger.debug('Could not send event to %s, probably stale socket.',
                                  client_addr)
                     # Maybe consider removing the client_addr from every topic?
                     self.subscribe_client_topic(client_addr, topic, False)
@@ -410,7 +456,7 @@ class AccessoryDriver:
             self.accumulated_qsize += self.event_queue.qsize()
 
             if self.sent_events > self.NUM_EVENTS_BEFORE_STATS:
-                logger.debug("Average queue size for the past %s events: %.2f",
+                logger.debug('Average queue size for the past %s events: %.2f',
                              self.sent_events, self.accumulated_qsize / self.sent_events)
                 self.sent_events = 0
                 self.accumulated_qsize = 0
@@ -465,7 +511,10 @@ class AccessoryDriver:
         logger.info("Paired with %s.", client_uuid)
         self.state.add_paired_client(client_uuid, client_public)
         self.persist()
-        self.update_advertisement()
+        # Safe mode added to avoid error during pairing, see
+        # https://github.com/home-assistant/home-assistant/issues/14567
+        if not self.safe_mode:
+            self.update_advertisement()
         return True
 
     def unpair(self, client_uuid):
@@ -480,7 +529,8 @@ class AccessoryDriver:
         logger.info("Unpairing client %s.", client_uuid)
         self.state.remove_paired_client(client_uuid)
         self.persist()
-        self.update_advertisement()
+        if not self.safe_mode:
+            self.update_advertisement()
 
     def setup_srp_verifier(self):
         """Create an SRP verifier for the accessory's info."""
@@ -517,6 +567,7 @@ class AccessoryDriver:
         hap_rep = self.accessory.to_HAP()
         if not isinstance(hap_rep, list):
             hap_rep = [hap_rep, ]
+        logger.debug("Get accessories response: %s", hap_rep)
         return {HAP_REPR_ACCS: hap_rep}
 
     def get_characteristics(self, char_ids):
@@ -552,6 +603,7 @@ class AccessoryDriver:
                 rep[HAP_REPR_STATUS] = SERVICE_COMMUNICATION_FAILURE
 
             chars.append(rep)
+        logger.debug("Get chars response: %s", chars)
         return {HAP_REPR_CHARS: chars}
 
     def set_characteristics(self, chars_query, client_addr):
@@ -571,45 +623,22 @@ class AccessoryDriver:
            }
 
         :type chars_query: dict
-
-        :return: Response status for each characteristic. For example:
-
-        .. code-block:: python
-
-           {
-              "characteristics": [{
-                 "aid": 1,
-                 "iid": 2,
-                 "status": 0,
-              }]
-           }
-
-        :rtype: dict
         """
-        chars_query = chars_query[HAP_REPR_CHARS]
-        chars_response = []
-        for cq in chars_query:
+        # TODO: Add support for chars that do no support notifications.
+        for cq in chars_query[HAP_REPR_CHARS]:
             aid, iid = cq[HAP_REPR_AID], cq[HAP_REPR_IID]
             char = self.accessory.get_characteristic(aid, iid)
 
             if HAP_PERMISSION_NOTIFY in cq:
                 char_topic = get_topic(aid, iid)
+                logger.debug('Subscribed client %s to topic %s',
+                             client_addr, char_topic)
                 self.subscribe_client_topic(
                     client_addr, char_topic, cq[HAP_PERMISSION_NOTIFY])
 
-            response = {
-                HAP_REPR_AID: aid,
-                HAP_REPR_IID: iid,
-                HAP_REPR_STATUS: CHAR_STAT_OK,
-            }
             if HAP_REPR_VALUE in cq:
                 # TODO: status needs to be based on success of set_value
                 char.client_update_value(cq[HAP_REPR_VALUE])
-                if "r" in cq:
-                    response[HAP_REPR_VALUE] = char.get_value()
-
-            chars_response.append(response)
-        return {HAP_REPR_CHARS: chars_response}
 
     def signal_handler(self, _signal, _frame):
         """Stops the AccessoryDriver for a given signal.
